local Recount2 = Recount2
local combatants
function Recount2:InitTracker()
	combatants = self.Data.Combatants
end

local INFO_DAMAGE	= 0x01
local INFO_HEAL		= 0x02
local INFO_EXTRA_ATTACK	= 0x04
local INFO_ENVIRONMENT	= 0x08
local INFO_POWERGAIN	= 0x10
local INFO_POWERDRAIN	= 0x20

local POWER_MANA	= 0x01
local POWER_RAGE	= 0x02
local POWER_ENERGY	= 0x04
local POWER_FOCUS	= 0x08

local SCHOOL_PHYSICAL	= 0x01
local SCHOOL_HOLY	= 0x02
local SCHOOL_FIRE	= 0x04
local SCHOOL_NATURE	= 0x08
local SCHOOL_FROST	= 0x10
local SCHOOL_SHADOW	= 0x20
local SCHOOL_ARCANE	= 0x40

local TYPE_HIT		= 0x00001
local TYPE_CRIT		= 0x00002
local TYPE_MISS		= 0x00004
local TYPE_DODGE	= 0x00008
local TYPE_PARRY	= 0x00010
local TYPE_BLOCK	= 0x00020
local TYPE_GLANCING	= 0x00040
local TYPE_CRUSHING	= 0x00080
local TYPE_TICK		= 0x00100
local TYPE_RESIST	= 0x00200
local TYPE_LEECH	= 0x00400
local TYPE_DRAIN	= 0x00800
local TYPE_SPLIT	= 0x01000
local TYPE_ABSORB	= 0x02000
local TYPE_EVADE	= 0x04000
local TYPE_IMMUNE	= 0x08000
local TYPE_REFLECT	= 0x10000



local TYPE_FLAGS = Recount2.BitFlags.Type



local CombatTable
local BitFlags = Recount2.BitFlags

local eventType = {
	["SWING_DAMAGE"] = "Damage",
	["RANGE_DAMAGE"] = "Damage",
	["SPELL_DAMAGE"] = "Damage",
	["SPELL_PERIODIC_DAMAGE"] = "Damage",
	["DAMAGE_SHIELD"] = "Damage",
	["DAMAGE_SPLIT"] = "Damage",
	["ENVIRONMENTAL_DAMAGE"] = "Damage",
	["SWING_MISSED"] = "Misses",
	["RANGE_MISSED"] = "Misses",
	["SPELL_MISSED"] = "Misses",
	["SPELL_PERIODIC_MISSED"] = "Misses",
	["DAMAGE_SHIELD_MISSED"] = "Misses",
	["SPELL_HEAL"] = "Healing",
	["SPELL_PERIODIC_HEAL"] = "Healing",
	["SPELL_ENERGIZE"] = "Power Gain",
	["SPELL_PERIODIC_ENERGIZE"] = "Power Gain",}

local affiliationLookup = {
	[0x1] = 'Mine',
	[0x2] = 'Party',
	[0x4] = 'Raid',
	[0x8] = 'Others',
}

local bit_band = bit.band
local string_sub = string.sub


local function FixCaps(capsstr)
	if type(capsstr)=="string" then
		return string.upper(string.sub(capsstr,1,1))..string.lower(string.sub(capsstr,2))
	else
		return nil
	end
end

function Recount2:CombatLogEvent(event, timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
	--First check to see if it passes the processing filters
	if Recount2.filterEvents[eventType[eventtype] or 'None']==nil and eventtype~="SPELL_CAST_START" then
		return
	end

	local srcAffil = bit_band(srcFlags, 0xf)
	local dstAffil = bit_band(dstFlags, 0xf)

	if Recount2.filterAffiliation[affiliationLookup[srcAffil] or 'Others']==nil and Recount2.filterAffiliation[affiliationLookup[dstAffil] or 'Others']==nil then
		return
	end

	if eventtype=="SPELL_CAST_START" then
		spellId = select(1, ...)
		if self.LastActive[srcGUID]==nil then
			self.LastActive[srcGUID]=Recount2:GetTable()
		end
		self.LastActive[srcGUID].spellId = spellId
		self.LastActive[srcGUID].time = timestamp
		return
	end

	local srcSpecial = false
	if srcName == nil then
		srcName = "No One"

		if string_sub(eventtype, 1, 13)=="ENVIRONMENTAL" then
			srcName = FixCaps(select(1, ...))
			srcGUID = "0xEnviro"..srcName
			srcName = srcName.." (Environmental)"
			srcSpecial = true
		elseif string_sub(eventtype, 1, 5)~="SWING" then
			srcName = select(2, ...)
			srcGUID = "0xSpell"..srcName
			srcSpecial = true
		end
	end
	if dstName == nil then
		dstName = "No One"
	end

	Recount2:UpdateGUIDInfo(srcGUID, srcName, srcFlags, timestamp, srcSpecial)
	Recount2:UpdateGUIDInfo(dstGUID, dstName, dstFlags, timestamp)

	if CombatTable[eventtype] then	
		--Did this event put our group into combat?
		if not self.inCombat then
			Recount2:CheckCombatStatus()
		end
		CombatTable[eventtype](Recount2, timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
	end
end


function Recount2:SwingDamage(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, amount, overkill, school, resisted, blocked, absorbed, critical, glancing, crushing)
	if combatants[srcGUID]==nil then
		self:VerifyGUID(srcGUID)
	end
	combatants[srcGUID].lastSwingSchool = school

	self:SpellDamage(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, nil, "Melee", school, amount, overkill, school, resisted, blocked, absorbed, critical, glancing, crushing)
end

function Recount2:SpellDamage(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellId, spellName, spellSchool, amount, overkill, school, resisted, blocked, absorbed, critical, glancing, crushing)
	HitType=TYPE_HIT
	if critical then
		HitType=TYPE_CRIT
	end
	if eventtype == "SPELL_PERIODIC_DAMAGE" then
		HitType=TYPE_TICK
	end
	if eventtype == "DAMAGE_SPLIT" then
		HitType=TYPE_SPLIT
	end
	if crushing then
		HitType=TYPE_CRUSHING
	end
	if glancing then
		HitType=TYPE_GLANCING
	end
	if blocked then
		HitType=HitType+TYPE_BLOCK
	end
	if absorbed then
		HitType=HitType+TYPE_ABSORB
	end
	
	self:AddToBuffer(timestamp, srcGUID, dstGUID, spellId, spellName, INFO_DAMAGE, 0, spellSchool, HitType, amount, resisted, absorbed, blocked, overkill)
end

function Recount2:EnvironmentalDamage(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags,enviromentalType, amount, overkill, school, resisted, blocked, absorbed, critical, glancing, crushing)
	HitType=TYPE_HIT
	if critical then
		HitType=TYPE_CRIT
	end
	if crushing then
		HitType=TYPE_CRUSHING
	end
	if glancing	then
		HitType=TYPE_GLANCING
	end
	if blocked then
		HitType=HitType+TYPE_BLOCK
	end
	if absorbed then
		HitType=HitType+TYPE_ABSORB
	end

	self:AddToBuffer(timestamp, srcGUID, dstGUID, nil, FixCaps(enviromentalType), INFO_DAMAGE+INFO_ENVIRONMENT, 0, school, HitType, amount, resisted, absorbed, blocked, overkill)
end
function Recount2:SwingMissed(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags,missType)
	if TYPE_FLAGS[missType] then
		HitType=TYPE_FLAGS[missType]
	else
		self:Print("Missing Swing Type",missType)
		return
	end

	self:AddToBuffer(timestamp, srcGUID, dstGUID, nil, "Melee", INFO_DAMAGE, 0, combatants[srcGUID].lastSwingSchool or 0, HitType, nil, nil, nil, nil)
end

function Recount2:SpellMissed(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags,spellId, spellName, spellSchool, missType)
	if TYPE_FLAGS[missType] then
		HitType=TYPE_FLAGS[missType]
	else
		self:Print("Missing Spell Type",missType)
		return
	end

	self:AddToBuffer(timestamp, srcGUID, dstGUID, spellId, spellName, INFO_DAMAGE, 0, spellSchool, HitType, nil, nil, nil, nil)
end

function Recount2:SpellHeal(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellId, spellName, spellSchool, amount, overheal, critical)
	local healtype=TYPE_HIT
	if critical then
		healtype=TYPE_CRIT
	end

	if eventtype == "SPELL_PERIODIC_HEAL" then
		healtype=TYPE_TICK
	end

	self:AddToBuffer(timestamp, srcGUID, dstGUID, spellId, spellName, INFO_HEAL, 0, spellSchool, healtype, amount, nil, nil, nil, overheal)
end


function Recount2:SpellEnergize(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellId, spellName, spellSchool, amount, powerType)
	if BitFlags.Power[powerType]==nil then
		self:Print("Unknown Power Type", spellName, powerType)
	end

	self:AddToBuffer(timestamp, srcGUID, dstGUID, spellId, spellName, INFO_POWERGAIN, BitFlags.Power[powerType], spellSchool, 0, amount, nil, nil, nil)
end

function Recount2:SpellExtraAttacks(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags,spellId, spellName, spellSchool, amount)
--[[	source = Recount.curr_srcName
	victim = Recount.curr_dstName

	local healtype="Hit"

	Recount:Print(Recount.curr_type.." "..spellName.." "..amount)
	Recount:AddDamageData(source, victim, spellName, Recount.SpellSchoolName[spellSchool], HitType, amount)--]]

	-- Elsia: Don't have use for extra attacks currently, amount is number of extra attacks it seems from combat log traces.
end

function Recount2:SpellDrainLeech(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags,spellId, spellName, spellSchool, amount, powerType, extraAmount, tmp1, tmp2)
	if eventtype == "SPELL_DRAIN" or eventtype == "SPELL_PERIODIC_DRAIN" then
		hitType = TYPE_DRAIN
	elseif eventtype == "SPELL_LEECH" or eventtype == "SPELL_PERIODIC_LEECH" then
		hitType = TYPE_LEECH
	end
end

CombatTable = {
	["SWING_DAMAGE"] = Recount2.SwingDamage,
	["RANGE_DAMAGE"] = Recount2.SpellDamage,
	["SPELL_DAMAGE"] = Recount2.SpellDamage,
	["SPELL_PERIODIC_DAMAGE"] = Recount2.SpellDamage,
	["DAMAGE_SHIELD"] = Recount2.SpellDamage,
	["DAMAGE_SPLIT"] = Recount2.SpellDamage,
	["ENVIRONMENTAL_DAMAGE"] = Recount2.EnvironmentalDamage,
	["SWING_MISSED"] = Recount2.SwingMissed,
	["RANGE_MISSED"] = Recount2.SpellMissed,
	["SPELL_MISSED"] = Recount2.SpellMissed,
	["SPELL_PERIODIC_MISSED"] = Recount2.SpellMissed,
	["DAMAGE_SHIELD_MISSED"] = Recount2.SpellMissed,
	["SPELL_HEAL"] = Recount2.SpellHeal,
	["SPELL_PERIODIC_HEAL"] = Recount2.SpellHeal,
	["SPELL_ENERGIZE"] = Recount2.SpellEnergize,
	["SPELL_PERIODIC_ENERGIZE"] = Recount2.SpellEnergize,
	--["SPELL_EXTRA_ATTACKS"] = Recount2.SpellExtraAttacks,
	--[[["SPELL_DRAIN"] = Recount2.SpellDrainLeech,
	["SPELL_LEECH"] = Recount2.SpellDrainLeech,
	["SPELL_PERIODIC_DRAIN"] = Recount2.SpellDrainLeech,
	["SPELL_PERIODIC_LEECH"] = Recount2.SpellDrainLeech,]]
}

